#include "mineuiview.h"

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

#include <qpainter.h>
#include <qtimer.h>
#include <gamecontrol/GameMisc.h>

#include "uidrawcontextkde.h"
#include "scorenamedlg.h"
#include "halloffamedlg.h"

#include "minefieldtooltip.h"

MineUIView::MineUIView(QWidget *parent=0, const char *name=0)
   : QWidget(parent, name),
     m_bFullUpdate(true),
     m_bUserMayEnterHallOfFame(true)
{
   m_gameMgr.init(&m_mouse);
   for(int i=0;i<4;i++) m_vbPressedButtons[i] = false;

   setBackgroundMode(NoBackground);

   // XXX i need a timer event only when the game is running XXX
	 m_pTimer = new QTimer(this);
	 QObject::connect(m_pTimer, SIGNAL(timeout()), this, SLOT(updateTimeSlot()));
	 m_pTimer->start(500, false);
	
	 // for tooltips
	 setMouseTracking(true);
	 new MineFieldToolTip(this, &m_gameMgr);
}

MineUIView::~MineUIView()
{
}

void MineUIView::setLevelMgr(CLevelMgr *pLevelMgr)
{
	m_pLevelMgr = pLevelMgr;
}

void MineUIView::solveSlot()
{
   m_gameMgr.showSolution();
   m_bUserMayEnterHallOfFame = false;
}	

void MineUIView::paintEvent( QPaintEvent *e )
{
   QPainter p;
   p.begin( this );

   {
	 	  CUIDrawContextKDE drawCtx(&p);
		  m_gameMgr.draw(&drawCtx, m_bFullUpdate);
	 }
	
   p.end();
   m_bFullUpdate = true;
}

void MineUIView::mousePressEvent(QMouseEvent *pEvt)
{
	 assert(m_pLevelMgr);
   CMouseInputDevice::MouseButton nButton = cnvMouseButton(pEvt->button());
	 if (nButton >= 0 && nButton < 4) m_vbPressedButtons[nButton] = true;

	 // XXX todo: activate timer only if game is running
	 //     (optimization) XXX
	
   // I'm using the mouse-button-enum as indices for array -> make sure that
   // the indices didn't change
   assert(CMouseInputDevice::BUTTON_RIGHT == 3);

   // if both left and right buttons are pushed, push the middle button instead
   if (   m_vbPressedButtons[CMouseInputDevice::BUTTON_LEFT]
       && m_vbPressedButtons[CMouseInputDevice::BUTTON_RIGHT])
			m_mouse.onPushButton(toUI(pEvt->pos()), CMouseInputDevice::BUTTON_MIDDLE);
	 else 	
		  m_mouse.onPushButton(toUI(pEvt->pos()), nButton);

   repaintChanges();
}

void MineUIView::mouseReleaseEvent(QMouseEvent *pEvt)
{
   CMouseInputDevice::MouseButton nButton = cnvMouseButton(pEvt->button());
   bool bIsRunningOld = m_gameMgr.isGameRunning();

	 m_mouse.onReleaseButton(toUI(pEvt->pos()));
	 repaintChanges();
	
   if (bIsRunningOld && !m_gameMgr.isGameRunning())
   {
      if (m_gameMgr.isGameWon())
      {
         checkGameResult();
      }
   }
   	
	 if (nButton >= 0 && nButton < 4) m_vbPressedButtons[nButton] = false;
}

void MineUIView::mouseMoveEvent(QMouseEvent *pEvt)
{
  m_mouse.onMoveMouse(toUI(pEvt->pos()));
  repaintChanges();
}

CMouseInputDevice::MouseButton MineUIView::cnvMouseButton(int nQtMouseButton)
{
	switch (nQtMouseButton) {
		case LeftButton: return  CMouseInputDevice::BUTTON_LEFT;
		case RightButton: return CMouseInputDevice::BUTTON_RIGHT;
		case MidButton: return CMouseInputDevice::BUTTON_MIDDLE;
		case NoButton:
		default: return CMouseInputDevice::BUTTON_NONE;
	}	
}


QSize MineUIView::getClientSize()
{
   return toQt(m_gameMgr.getClientWindowSize());
}

// XXX don't know if needed ??? XXX
QSize MineUIView::sizeHint()
{
	return getClientSize();
}

void MineUIView::reset()
{
   m_gameMgr.resetGame();
   repaint();
}

void MineUIView::setGameDim(const CGameDim &gameDim)
{
   m_gameMgr.resize(gameDim);
   m_bUserMayEnterHallOfFame = true;	
}

CGameDim MineUIView::getGameDim() const
{
	return m_gameMgr.getGameDim();
}

void MineUIView::repaintChanges()
{
	m_bFullUpdate = false;
	repaint();
}

void MineUIView::updateTimeSlot()
{
	repaintChanges();
}

GameConstant::GameLevel MineUIView::getCurrentGameLevel()
{
   CGameDim gameDim = m_gameMgr.getGameDim();
   GameConstant::GameLevel nGameLevel(GameConstant::LEVEL_USER_DEFINED);

   if (gameDim == GameConstant::getGameDimOfLevel(GameConstant::LEVEL_BEGINNER))
      nGameLevel = GameConstant::LEVEL_BEGINNER;

   if (gameDim == GameConstant::getGameDimOfLevel(GameConstant::LEVEL_INTERMEDIATE))
      nGameLevel = GameConstant::LEVEL_INTERMEDIATE;

   if (gameDim == GameConstant::getGameDimOfLevel(GameConstant::LEVEL_EXPERT))
      nGameLevel = GameConstant::LEVEL_EXPERT;

	 return nGameLevel;
}


void MineUIView::checkGameResult()
{
   int nTime = m_gameMgr.getTime();

   if (m_pLevelMgr->hasReachedNewHighScore(nTime) && m_bUserMayEnterHallOfFame)
   {
      ScoreNameDlg *pNameDlg = new ScoreNameDlg(this);
      if (pNameDlg->exec() == ScoreNameDlg::Accepted)
      {
         m_pLevelMgr->enterHallOfFame(std::string(pNameDlg->getName()), nTime);

         CScoreTable scoreTable;
         m_pLevelMgr->getScoreTable(scoreTable);
         HallOfFameDlg *pDlg = new HallOfFameDlg(this, scoreTable);
	       pDlg->exec();
	       delete pDlg;
      }
			delete pNameDlg;
   }
}


